package com.dragon.tj.portal.auth.service;

import com.dragon.tj.portal.auth.client.DcucAuthClient;
import com.dragon.tj.portal.auth.model.AuthReq;
import com.dragon.tj.portal.auth.model.AuthResult;
import com.dragon.tj.portal.auth.model.RR;
import com.dragon.tj.portal.auth.web.entity.SysUser;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;
import org.springframework.util.CollectionUtils;
import retrofit2.Call;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Objects;
import java.util.Set;

/**
 * 用户权限处理
 *
 * @author tienchin
 */
@Component
public class SysPermissionService {
    private static final Logger LOGGER = LoggerFactory.getLogger(SysPermissionService.class);

    @Autowired
    private DcucAuthClient dcucAuthClient;

    @Value("${app.code}")
    private String appCode;

    /**
     * 获取数据权限
     *
     * @param user 用户信息
     * @return 数据权限信息
     */
    public Set<String> getDataPermission(SysUser user) {
        return new HashSet<>();
    }

    /**
     * 获取菜单权限
     *
     * @param user 用户信息
     * @return 菜单权限信息
     */
    public Set<String> getMenuPermission(SysUser user) {
        Set<String> perms = new HashSet<>();
        String idCard = user.getIdcard();
        try {
            LOGGER.info("登录用户[{}]调用权限管理服务-功能级鉴权接口", idCard);
            Call<RR<List<AuthResult>>> call = dcucAuthClient.getAuthFunctions(appCode, idCard, new AuthReq(appCode, idCard));
            RR<List<AuthResult>> body = call.execute().body();
            if (Objects.nonNull(body)) {
                if ("200".equals(body.getStatusCode())) {
                    List<AuthResult> results = body.getResult();
                    perms.addAll(extractPermissions(results));
                } else {
                    LOGGER.error("登录用户[{}]调用权限管理服务-功能级鉴权接口 出错: {}", idCard, body.getMessage());
                }
            } else {
                LOGGER.error("登录用户[{}]调用权限管理服务-功能级鉴权接口 出错: 返回为空", idCard);
            }
        } catch (Exception e) {
            LOGGER.error("登录用户[{}]调用权限管理服务-功能级鉴权接口 出错", idCard, e);
        }

        return perms;
    }

    private List<String> extractPermissions(List<AuthResult> results) {
        List<String> permissions = new ArrayList<>();
        if (!CollectionUtils.isEmpty(results)) {
            for (AuthResult result : results) {
                permissions.add(result.getCode());
                List<AuthResult> nextCode = result.getNextCode();
                if (!CollectionUtils.isEmpty(nextCode)) {
                    permissions.addAll(extractPermissions(nextCode));
                }
            }
        }
        return permissions;
    }
}
