package com.dragon.tj.portal.auth.web.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.OrderItem;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.dragon.tj.portal.auth.client.DcucUserClient;
import com.dragon.tj.portal.auth.model.RR;
import com.dragon.tj.portal.auth.web.entity.SysUser;
import com.dragon.tj.portal.auth.web.mapper.SysUserMapper;
import com.dragon.tj.portal.auth.web.service.SysUserService;
import com.dragon.tj.portal.entity.PageParam;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import retrofit2.Call;

import java.util.Objects;

/**
 * <p>
 * 用户信息表 服务实现类
 * </p>
 *
 * @author caiaa
 * @since 2023-06-23
 */
@Service
public class SysUserServiceImpl extends ServiceImpl<SysUserMapper, SysUser> implements SysUserService {
    private static final Logger LOGGER = LoggerFactory.getLogger(SysUserServiceImpl.class);

    @Autowired
    private DcucUserClient dcucUserClient;

    @Value("${app.code}")
    private String appCode;

    @Override
    public SysUser getUserById(String idCard) {
        SysUser sysUser = getById(idCard);
        if (Objects.isNull(sysUser)) {
            LOGGER.info("登录用户[{}]调用用户管理服务-获取警员详情接口", idCard);
            sysUser = getUserFromDcuc(idCard);
        }
        return sysUser;
    }

    @Override
    public Page<SysUser> list(PageParam<SysUser> pageParam) {
        Page<SysUser> page = new Page<>(pageParam.getPage(), pageParam.getSize());
        String order = pageParam.getOrder();
        if (StringUtils.isNotBlank(order)) {
            page.addOrder(OrderItem.asc(order));
        }

        SysUser sysUser = pageParam.getParams();
        LambdaQueryWrapper<SysUser> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.like(StringUtils.isNotEmpty(sysUser.getName()), SysUser::getName, sysUser.getName());
        queryWrapper.like(StringUtils.isNotEmpty(sysUser.getPoliceNumber()), SysUser::getPoliceNumber, sysUser.getPoliceNumber());
        queryWrapper.eq(sysUser.getOrgCode() != null, SysUser::getOrgCode, sysUser.getOrgCode());
        queryWrapper.eq(SysUser::getDeleted, "0");
        return baseMapper.selectPage(page, queryWrapper);
    }

    public SysUser getUserFromDcuc(String idCard) {
        SysUser sysUser = null;
        try {
            Call<RR<SysUser>> call = dcucUserClient.getUserInfo(appCode, idCard, idCard);
            RR<SysUser> body = call.execute().body();
            if (Objects.nonNull(body)) {
                if ("200".equals(body.getStatusCode())) {
                    sysUser = body.getResult();
                } else {
                    LOGGER.error("登录用户[{}]调用用户管理服务-获取警员详情接口 出错: {}", idCard, body.getMessage());
                }
            } else {
                LOGGER.error("登录用户[{}]调用用户管理服务-获取警员详情接口 出错: 返回为空", idCard);
            }
        } catch (Exception e) {
            LOGGER.error("登录用户[{}]调用用户管理服务-获取警员详情接口 出错", idCard, e);
        }
        return sysUser;
    }

}
