package com.dragon.tj.portal.controller;

import com.dragon.tj.portal.auth.model.LoginUser;
import com.dragon.tj.portal.auth.util.SecurityUtils;
import com.dragon.tj.portal.common.base.R;
import com.dragon.tj.portal.common.enums.log.ModuleEnum;
import com.dragon.tj.portal.component.log.annotation.SysLog;
import com.dragon.tj.portal.entity.AppInfo;
import com.dragon.tj.portal.entity.InstallInfo;
import com.dragon.tj.portal.entity.PageParam;
import com.dragon.tj.portal.service.AppService;
import com.dragon.tj.portal.service.InstallInfoService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.EnableScheduling;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import java.time.LocalDateTime;
import java.util.List;

@RestController
@RequestMapping("/app")
@EnableScheduling
public class AppController {


    @Autowired
    private AppService appService;

    @Autowired
    private InstallInfoService installInfoService;

    /*
     *    1. 我的应用查询 批量排序收藏
     * */
    @SysLog(value = "我的应用查询 批量排序收藏", module = ModuleEnum.APP_QUERY)
    @PostMapping("list")
    public R appList(@RequestBody PageParam<AppInfo> appInfoPage) {
        LoginUser loginUser = SecurityUtils.getLoginUser();
        return R.ok(appService.installApp(appInfoPage, loginUser));
    }

    /*
     *  1-1安装应用
     * */
    @SysLog(value = "收藏应用", module = ModuleEnum.APP_COLLECTION)
    @GetMapping("install/{appId}")
    public R install(@PathVariable("appId") Integer appId) {
        LoginUser loginUser = SecurityUtils.getLoginUser();
        return R.ok(appService.install(appId, loginUser.getIdCard()));
    }

    /*
     *  1-2卸载应用
     * */
    @SysLog(value = "取消收藏应用", module = ModuleEnum.APP_COLLECTION)
    @GetMapping("uninstall/{appId}")
    public R uninstall(@PathVariable("appId") Integer appId) {
        LoginUser loginUser = SecurityUtils.getLoginUser();
        return R.ok(appService.uninstall(appId, loginUser.getIdCard()));
    }

    /***************************************/

    /*
     *    2. 类型查询
     * */
    @GetMapping("types")
    public R getAppType() {
        return appService.getType();
    }

    /*
     *    3. 应用 CRUD
     * */
    @SysLog(value = "应用添加", module = ModuleEnum.APP_ADD)
    @PostMapping("add")
    public R addApp(@RequestBody AppInfo appInfo) {
        System.out.println(appInfo);
        LoginUser loginUser = SecurityUtils.getLoginUser();
        return R.ok(appService.add(appInfo, loginUser));
    }

    @SysLog(value = "应用删除", module = ModuleEnum.APP_DELETE)
    @GetMapping("delete/{id}")
    public R delete(@PathVariable("id") Long id) {
        return R.ok(appService.delete(id));
    }

    @SysLog(value = "应用更新", module = ModuleEnum.APP_UPDATE)
    @PostMapping("update")
    public R update(@RequestBody AppInfo appInfo) {
        appInfo.setUpdateTime(LocalDateTime.now());
        return R.ok(appService.update(appInfo));
    }

    /*
     *   批量更新收藏
     * */
    @SysLog(value = "批量更新收藏", module = ModuleEnum.APP_UPDATE)
    @PostMapping("batchUpdate")
    public R batchUpdate(@RequestBody List<InstallInfo> list) {
        installInfoService.batchUpdate(list, 0);
        return R.ok();
    }

    //    @SysLog("通过ID获取应用")
    @SysLog(value = "通过ID获取应用", module = ModuleEnum.APP_QUERY)
    @GetMapping("info/{id}")
    public R detail(@PathVariable("id") Long id) {
        return R.ok(appService.detail(id));
    }

    /*
     *   通过应用名称,应用事权单位名称搜索app
     * */
    @SysLog(value = "通过应用名称,应用事权单位名称搜索app", module = ModuleEnum.APP_QUERY)
    @PostMapping("search")
    public R search(@RequestBody PageParam<AppInfo> appInfoPage) {

        return R.ok(appService.search(appInfoPage));
    }

    @SysLog(value = "批量添加应用", module = ModuleEnum.APP_ADD)
    @PostMapping("batchAdd")
    public R batchAdd(MultipartFile file) {
        boolean result = appService.batchAdd(file);
        return result ? R.ok() : R.failed();
    }


    /*
     *   点击量
     * */
    @GetMapping("clickIncr/{id}")
    public R clickIncr(@PathVariable("id") Long id) {

        return R.ok(appService.clickIncr(id));
    }


    /*
     *    4. 热门应用统计 定时、实时？
     * */
    // @Scheduled(fixedDelay = 1000 * 60 * 60)
    public void appStatHandler() {

        appService.updateAppStat();
    }

    /*
     *    5. 专项工具为应用类型的一种
     * */
    /**********************/

    /*
     * 上传图片
     * */


}

