package com.dragon.tj.portal.service.impl;

import cn.hutool.core.collection.CollUtil;
import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.dragon.tj.portal.auth.model.LoginUser;
import com.dragon.tj.portal.auth.util.SecurityUtils;
import com.dragon.tj.portal.auth.web.entity.SysUser;
import com.dragon.tj.portal.auth.web.service.SysDeptService;
import com.dragon.tj.portal.common.base.R;
import com.dragon.tj.portal.common.constants.BusinessConstants;
import com.dragon.tj.portal.common.convert.mesage.MessageInfoVOConvert;
import com.dragon.tj.portal.common.convert.mesage.MessageReqConvert;
import com.dragon.tj.portal.common.dto.message.MessageInfoReq;
import com.dragon.tj.portal.common.dto.message.MessageInfoSend;
import com.dragon.tj.portal.common.dto.message.MessageInfoUserInfo;
import com.dragon.tj.portal.common.enums.message.MessageInfoErrorEnums;
import com.dragon.tj.portal.common.enums.message.MsgRecordTypeEnum;
import com.dragon.tj.portal.common.enums.message.ReadStatusEnum;
import com.dragon.tj.portal.common.enums.message.ScopeEnums;
import com.dragon.tj.portal.common.vo.message.*;
import com.dragon.tj.portal.component.exception.message.MessageInfoException;
import com.dragon.tj.portal.component.message.KafkaInitialConfiguration;
import com.dragon.tj.portal.component.message.MessageProducer;
import com.dragon.tj.portal.entity.MessageInfo;
import com.dragon.tj.portal.entity.MessageInfoScope;
import com.dragon.tj.portal.entity.MsgRecord;
import com.dragon.tj.portal.mapper.MessageInfoMapper;
import com.dragon.tj.portal.mapper.NoticeInfoMapper;
import com.dragon.tj.portal.service.MessageInfoScopeService;
import com.dragon.tj.portal.service.MessageInfoService;
import com.dragon.tj.portal.service.MsgRecordService;
import com.google.common.collect.Lists;
import com.google.common.collect.Sets;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.codec.digest.DigestUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.time.LocalDateTime;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.Set;

/**
 * <p>
 * 信息表 服务实现类
 * </p>
 *
 * @author huey
 * @since 2023-06-15
 */
@Slf4j
@Service
public class MessageInfoServiceImpl extends ServiceImpl<MessageInfoMapper, MessageInfo> implements MessageInfoService {

    @Autowired
    private MessageProducer producer;

    @Autowired
    private MessageInfoScopeService messageInfoScopeService;

    @Autowired
    private SysDeptService sysDeptService;

    @Autowired
    private MessageReqConvert messageReqConvert;

    @Autowired
    private MessageInfoVOConvert messageInfoVOConvert;

    @Autowired
    private MsgRecordService msgRecordService;

    @Value("${message-secret:false}")
    private boolean messageSecret;

    @Autowired
    private NoticeInfoMapper noticeInfoMapper;

    @Override
    public R push(MessageInfoReq messageInfoReq, LoginUser loginUser) {
        return R.ok(this.doMessage(messageInfoReq, loginUser));
    }

    @Override
    public MessageInfoVO detail(Long id) {
        MessageInfo messageInfo = this.getById(id);
        return messageInfoVOConvert.toVo(messageInfo);
    }

    @Override
    public boolean update(MessageInfoParam messageInfoParam) {
        MessageInfo messageInfo = messageInfoVOConvert.paramTo(messageInfoParam);
        return this.updateById(messageInfo);
    }

    @Override
    public boolean read(Long id) {
        MessageInfoScope messageInfo = messageInfoScopeService.getById(id);
        Integer readStatus = messageInfo.getReadStatus();
        if (ReadStatusEnum.YES.value().equals(readStatus)) {
            throw new MessageInfoException(MessageInfoErrorEnums.Code.MESSAGE_READ_ALREADY);
        }
        messageInfo.setReadStatus(ReadStatusEnum.YES.value());
        return messageInfoScopeService.updateById(messageInfo);
    }

    @Override
    public boolean reads(List<Long> ids) {
        List<MessageInfoScope> list = Lists.newArrayList();
        ids.forEach(e -> {
            MessageInfoScope messageInfo = new MessageInfoScope();
            messageInfo.setReadStatus(ReadStatusEnum.YES.value());
            messageInfo.setId(e);

        });
        return messageInfoScopeService.updateBatchById(list);
    }

    @Override
    public IPage queryPage(Page page, MessagePageParam req) {
        String idCard = SecurityUtils.getLoginUser().getIdCard();
        IPage<MessageInfo> queryPage = baseMapper.queryPage(page, req,idCard);
        return queryPage;
    }

    @Override
    public MsgCountVO getCount() {
        LoginUser loginUser = SecurityUtils.getLoginUser();
        String idCard = loginUser.getIdCard();
        MsgCountVO msgCountVO = new MsgCountVO();
        Integer noticeY = noticeInfoMapper.getCount(idCard, ReadStatusEnum.YES.value());
        Integer noticeN = noticeInfoMapper.getCount(idCard, ReadStatusEnum.NO.value());

        Integer msgY = this.baseMapper.getCount(idCard, ReadStatusEnum.YES.value());
        Integer msgN = this.baseMapper.getCount(idCard, ReadStatusEnum.NO.value());

        log.info("getCount-{}-{}-{}-{}-{}", idCard, noticeN, noticeY, msgN, msgY);
        msgCountVO.setNoticeNCount(noticeN);
        msgCountVO.setNoticeYCount(noticeY);
        msgCountVO.setMessageYCount(msgY);
        msgCountVO.setMessageNCount(msgN);
        return msgCountVO;
    }

    @Override
    public List<ContactMemberVO> getContactsMemberList(Integer limit) {
        LoginUser loginUser = SecurityUtils.getLoginUser();
        String idCard = loginUser.getIdCard();
        List<ContactMemberVO> list = this.baseMapper.getContactsMemberList(idCard,limit);
        return list;
    }


    /**
     * @author huey China.
     * @Description :
     * @Date Created in 2023/6/16 11:44
     */
    private boolean transferIds(MessageInfoReq messageInfoReq, LoginUser loginUser) {
        // 120101510000 120000450200
        //120222197001010001 120222197001010002
        boolean isSend = false;
        Integer messageType = messageInfoReq.getMessageType();
        Set<String> scopeIds = messageInfoReq.getScopeIds();
        MessageInfoUserInfo messageInfoUserInfo = this.getMessageClientIds(messageType, scopeIds);

        if (CollUtil.isNotEmpty(messageInfoUserInfo.getClientIds())) {
            MessageInfoSend messageInfoSend = this.doSaveMessageInfo(messageInfoReq, messageInfoUserInfo, loginUser);
            isSend = producer.send(KafkaInitialConfiguration.sseTopic, JSON.toJSONString(messageInfoSend));
            if (isSend) {
                MsgRecord msgRecord = new MsgRecord();
                msgRecord.setCreateBy("admin");
                msgRecord.setCreateTime(LocalDateTime.now());
                msgRecord.setReqContent(JSON.toJSONString(messageInfoSend));
                msgRecord.setType(MsgRecordTypeEnum.MESSAGE.value());
                msgRecordService.save(msgRecord);
            }
        }
        return isSend;
    }

    @Transactional(rollbackFor = Exception.class)
    public MessageInfoSend doSaveMessageInfo(MessageInfoReq messageInfoReq, MessageInfoUserInfo messageInfoUserInfo, LoginUser loginUser) {
        Set<String> clientIds = messageInfoUserInfo.getClientIds();
        log.info("doSaveMessageInfo-start-{}-{}", loginUser.getIdCard(), clientIds);
        Integer messageType = messageInfoReq.getMessageType();
        ScopeEnums scopeEnums = ScopeEnums.ofMessageType(messageType);
        List<MessageInfoScope> insertList = Lists.newArrayList();
        MessageInfoSend messageInfoSend = new MessageInfoSend();
        messageInfoSend.setMessageInfoReq(messageInfoReq);
        messageInfoSend.setClientIds(clientIds);

        MessageInfo messageInfo = messageReqConvert.reqToInfo(messageInfoReq);
        messageInfo.setCreateBy(loginUser.getIdCard());
        messageInfo.setCreateUser(loginUser.getUsername());
        SysUser user = loginUser.getUser();
        if (Objects.nonNull(user)) {
            messageInfo.setCreateOrgCode(loginUser.getUser().getOrgCode());
            messageInfo.setCreateOrgName(loginUser.getUser().getOrgName());
        }

        // 存储
        boolean isSaveBatchSuccess = false;
        try {
            isSaveBatchSuccess = this.save(messageInfo);

            messageInfoSend.setMessageInfo(messageInfo);
            Map<String, SysUser> userMap = messageInfoUserInfo.getUserMap();
            clientIds.forEach(e -> {
                MessageInfoScope messageInfoScope = new MessageInfoScope();
                messageInfoScope.setMessageInfoId(messageInfo.getId());
                messageInfoScope.setMemberId(e);
                SysUser sysUser = userMap.get(e);
                if (Objects.nonNull(sysUser) && StringUtils.isNotBlank(sysUser.getOrgCode())) {
                    messageInfoScope.setScopeId(sysUser.getOrgCode());
                    messageInfoScope.setScopeName(sysUser.getOrgName());
                }
                messageInfoScope.setScopeType(scopeEnums.value());
                insertList.add(messageInfoScope);
            });
            isSaveBatchSuccess = messageInfoScopeService.saveBatch(insertList);
        } catch (Exception e) {
            log.warn("消息中心 saveBatch 存储失败 {}",e);
        }

        // 批量存储失败，则逐个存储
        if (!isSaveBatchSuccess) {
            try {
                this.save(messageInfo);
                insertList.forEach(e -> {
                    messageInfoScopeService.save(e);
                });
            } catch (Exception e) {
                log.warn("消息中心 存储失败 {}");
            }
        }
        log.info("doSaveMessageInfo-end-{}-{}", loginUser.getIdCard(), clientIds);
        return messageInfoSend;
    }

    private MessageInfoUserInfo getMessageClientIds(Integer messageType, Set<String> scopeIds) {
        //登录人标识
        if (CollUtil.isEmpty(scopeIds)) {
            if (messageType.equals(BusinessConstants.DICT_ITEM_ID_2)) {
                throw new MessageInfoException(MessageInfoErrorEnums.Code.MESSAGE_CONTRACTS_EMPTY);
            } else {
                throw new MessageInfoException(MessageInfoErrorEnums.Code.MESSAGE_DEPT_EMPTY);
            }
        }

        MessageInfoUserInfo messageInfoUserInfo = sysDeptService.getAllUsers(scopeIds, messageType);
        return messageInfoUserInfo;
    }


    /**
     * @author huey China.
     * @Description : 预先处理 TODO message 没走字典
     * @Date Created in 2023/6/16 11:25
     */
    private boolean doMessage(MessageInfoReq messageInfoReq, LoginUser loginUser) {

        if (this.validate(messageInfoReq, loginUser.getIdCard())) {
            return this.transferIds(messageInfoReq, loginUser);
        }
        return false;
    }

    /**
     * @author huey China.
     * @Description : md5 验证消息有效性
     * @Date Created in 2023/6/16 16:15
     */
    private boolean validate(MessageInfoReq messageInfoReq, String idCard) {
        log.info("validate-start-{}", idCard);
        if (messageSecret) {
            String correctHash = DigestUtils.md5Hex(idCard + messageInfoReq.getT());
            if (!correctHash.equalsIgnoreCase(messageInfoReq.getK())) {
                return false;
            }
        }
        log.info("validate-end-{}", idCard);
        return true;
    }

}
